
# q-expansion of newform 864.2.a.d, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 864
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [3], [-6], [-3], [2], [-3], [-6], [8], [0], [7], [-8], [-12], [-6], [-4], [-6], [-1], [-3], [-12], [-15], [9], [12], [10], [9], [16], [15], [6], [6], [-14], [-12], [0], [14], [-21], [-4], [-3], [-14], [3], [18], [-8], [12], [9], [6], [-5], [-2], [15], [15], [-24], [-12], [-6], [8], [-12], [-3], [0], [20], [12], [10], [-15], [-18], [-8], [12], [2], [12], [-18], [-1], [4], [3], [9], [-12], [-1], [-20], [-18], [3], [23], [-27], [0], [-34], [2], [20], [21], [-30], [-15], [-30], [-2], [0], [12], [22], [-30], [2], [15], [-6], [-12], [15], [-42], [24], [42], [-10], [14], [9], [-15], [-9], [2], [24], [-46], [-3], [11], [-18], [-28], [-24], [-26], [3], [11], [-2], [9], [-27], [-4], [-36], [-24], [28], [12], [-5], [-1], [-2], [24], [12], [-46], [-51], [36], [24], [-18], [-12], [-6], [45], [-27], [34], [-49], [20], [-27], [40], [-44], [36], [-50], [9], [-6], [45], [36], [-53], [4], [27], [42], [-13], [14], [-9], [36], [-39], [-48], [24], [-28], [7], [-22], [-18], [-46], [33], [6], [-44], [6], [-21], [26]]
